@@ -29,16 +29,18 @@ from werkzeug.exceptions import HTTPException
 
 from moto import settings
 from moto.core.common_types import TYPE_IF_NONE, TYPE_RESPONSE
-from moto.core.exceptions import DryRunClientError
+from moto.core.exceptions import DryRunClientError, ServiceException
+from moto.core.serialize import SERIALIZERS, XFormedAttributePicker
 from moto.core.utils import (
     camelcase_to_underscores,
+    get_service_model,
     gzip_decompress,
     method_names_from_class,
     params_sort_function,
     utcfromtimestamp,
 )
 from moto.utilities.aws_headers import gen_amzn_requestid_long
-from moto.utilities.utils import load_resource, load_resource_as_bytes
+from moto.utilities.utils import get_partition, load_resource, load_resource_as_bytes
 
 log = logging.getLogger(__name__)
 
@@ -81,6 +83,29 @@ def _decode_dict(d: Dict[Any, Any]) -> Dict[str, Any]:
     return decoded
 
 
+@functools.lru_cache(maxsize=None)
+def _get_method_urls(service_name: str, region: str) -> Dict[str, Dict[str, str]]:
+    method_urls: Dict[str, Dict[str, str]] = defaultdict(dict)
+    service_name = boto3_service_name.get(service_name) or service_name  # type: ignore
+    conn = boto3.client(service_name, region_name=region)
+    op_names = conn._service_model.operation_names
+    for op_name in op_names:
+        op_model = conn._service_model.operation_model(op_name)
+        _method = op_model.http["method"]
+        request_uri = op_model.http["requestUri"]
+        if service_name == "route53" and request_uri.endswith("/rrset/"):
+            # Terraform 5.50 made a request to /rrset/
+            # Terraform 5.51+ makes a request to /rrset - so we have to intercept both variants
+            request_uri += "?"
+        if service_name == "lambda" and request_uri.endswith("/functions/"):
+            # AWS JS SDK behaves differently from other SDK's, does not send a trailing slash
+            request_uri += "?"
+        uri_regexp = BaseResponse.uri_to_regexp(request_uri)
+        method_urls[_method][uri_regexp] = op_model.name
+
+    return method_urls
+
+
 class DynamicDictLoader(DictLoader):
     def update(self, mapping: Dict[str, str]) -> None:
         self.mapping.update(mapping)  # type: ignore[attr-defined]
@@ -178,6 +203,7 @@ class ActionAuthenticatorMixin(object):
                 data=self.data,  # type: ignore[attr-defined]
                 body=self.body,  # type: ignore[attr-defined]
                 headers=self.headers,  # type: ignore[attr-defined]
+                action=self._get_action(),  # type: ignore[attr-defined]
             )
             iam_request.check_signature()
             iam_request.check_action_permitted(resource)
@@ -193,7 +219,7 @@ class ActionAuthenticatorMixin(object):
         self, bucket_name: Optional[str] = None, key_name: Optional[str] = None
     ) -> None:
         arn = f"{bucket_name or '*'}/{key_name}" if key_name else (bucket_name or "*")
-        resource = f"arn:aws:s3:::{arn}"
+        resource = f"arn:{get_partition(self.region)}:s3:::{arn}"  # type: ignore[attr-defined]
 
         from moto.iam.access_control import S3IAMRequest
 
@@ -246,6 +272,17 @@ class ActionAuthenticatorMixin(object):
         return decorator
 
 
+class ActionResult:
+    """Wrapper class for serializable results returned from `responses.py` methods."""
+
+    def __init__(self, result: object) -> None:
+        self._result = result
+
+    @property
+    def result(self) -> object:
+        return self._result
+
+
 class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
     default_region = "us-east-1"
     # to extract region, use [^.]
@@ -284,7 +321,7 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
         """
 
         @functools.wraps(to_call)  # type: ignore
-        def _inner(request: Any, full_url: str, headers: Any) -> TYPE_RESPONSE:
+        def _inner(request: Any, full_url: str, headers: Any) -> TYPE_RESPONSE:  # type: ignore[misc]
             response = getattr(cls(), to_call.__name__)(request, full_url, headers)
             if isinstance(response, str):
                 status = 200
@@ -345,6 +382,7 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
 
         if hasattr(self.body, "read"):
             self.body = self.body.read()
+        self.raw_body = self.body
 
         # https://github.com/getmoto/moto/issues/6692
         # Content coming from SDK's can be GZipped for performance reasons
@@ -408,6 +446,7 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
         self.data = querystring
         self.method = request.method
         self.region = self.get_region_from_url(request, full_url)
+        self.partition = get_partition(self.region)
         self.uri_match: Optional[re.Match[str]] = None
 
         self.headers = request.headers
@@ -481,37 +520,38 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
         self.setup_class(request, full_url, headers)
         return self.call_action()
 
-    def uri_to_regexp(self, uri: str) -> str:
+    @staticmethod
+    def uri_to_regexp(uri: str) -> str:
         """converts uri w/ placeholder to regexp
-          '/cars/{carName}/drivers/{DriverName}'
-        -> '^/cars/.*/drivers/[^/]*$'
+          '/accounts/{AwsAccountId}/namespaces/{Namespace}/groups'
+        -> '^/accounts/(?P<AwsAccountId>[^/]+)/namespaces/(?P<Namespace>[^/]+)/groups$'
 
-          '/cars/{carName}/drivers/{DriverName}/drive'
-        -> '^/cars/.*/drivers/.*/drive$'
+          '/trustStores/{trustStoreArn+}'
+        -> '^/trustStores/(?P<trustStoreArn>.+)$'
 
         """
 
-        def _convert(elem: str, is_last: bool) -> str:
+        def _convert(elem: str) -> str:
             if not re.match("^{.*}$", elem):
                 # URL-parts sometimes contain a $
                 # Like Greengrass: /../deployments/$reset
                 # We don't want to our regex to think this marks an end-of-line, so let's escape it
                 return elem.replace("$", r"\$")
+
+            # When the element ends with +} the parameter can contain a / otherwise not.
+            slash_allowed = elem.endswith("+}")
             name = (
                 elem.replace("{", "")
                 .replace("}", "")
                 .replace("+", "")
                 .replace("-", "_")
             )
-            if is_last:
-                return f"(?P<{name}>[^/]+)"
-            return f"(?P<{name}>.*)"
+            if slash_allowed:
+                return f"(?P<{name}>.+)"
+            return f"(?P<{name}>[^/]+)"
 
         elems = uri.split("/")
-        num_elems = len(elems)
-        regexp = "/".join(
-            [_convert(elem, (i == num_elems - 1)) for i, elem in enumerate(elems)]
-        )
+        regexp = "/".join([_convert(elem) for elem in elems])
         return f"^{regexp}$"
 
     def _get_action_from_method_and_request_uri(
@@ -521,22 +561,8 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
         You can refer to example from link below
         https://github.com/boto/botocore/blob/develop/botocore/data/iot/2015-05-28/service-2.json
         """
-
-        service_name = boto3_service_name.get(self.service_name) or self.service_name  # type: ignore
-        conn = boto3.client(service_name, region_name=self.region)
-
-        # make cache if it does not exist yet
-        if not hasattr(self, "method_urls"):
-            self.method_urls: Dict[str, Dict[str, str]] = defaultdict(
-                lambda: defaultdict(str)
-            )
-            op_names = conn._service_model.operation_names
-            for op_name in op_names:
-                op_model = conn._service_model.operation_model(op_name)
-                _method = op_model.http["method"]
-                uri_regexp = self.uri_to_regexp(op_model.http["requestUri"])
-                self.method_urls[_method][uri_regexp] = op_model.name
-        regexp_and_names = self.method_urls[method]
+        methods_url = _get_method_urls(self.service_name, self.region)
+        regexp_and_names = methods_url[method]
         for regexp, name in regexp_and_names.items():
             match = re.match(regexp, request_uri)
             self.uri_match = match
@@ -545,7 +571,9 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
         return None  # type: ignore[return-value]
 
     def _get_action(self) -> str:
-        action = self.querystring.get("Action", [""])[0]
+        action = self.querystring.get("Action")
+        if action and isinstance(action, list):
+            action = action[0]
         if action:
             return action
         # Some services use a header for the action
@@ -556,6 +584,18 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
         # get action from method and uri
         return self._get_action_from_method_and_request_uri(self.method, self.raw_path)
 
+    def serialized(self, action_result: ActionResult) -> TYPE_RESPONSE:
+        service_model = get_service_model(self.service_name)
+        operation_model = service_model.operation_model(self._get_action())
+        serializer_cls = SERIALIZERS[service_model.protocol]
+        serializer = serializer_cls(
+            operation_model=operation_model,
+            pretty_print=settings.PRETTIFY_RESPONSES,
+            value_picker=XFormedAttributePicker(),
+        )
+        serialized = serializer.serialize(action_result.result)
+        return serialized["status_code"], serialized["headers"], serialized["body"]  # type: ignore[return-value]
+
     def call_action(self) -> TYPE_RESPONSE:
         headers = self.response_headers
         if hasattr(self, "_determine_resource"):
@@ -579,6 +619,8 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
             method = getattr(self, action)
             try:
                 response = method()
+            except ServiceException as e:
+                response = ActionResult(e)  # type: ignore[assignment]
             except HTTPException as http_error:
                 response_headers: Dict[str, Union[str, int]] = dict(
                     http_error.get_headers() or []
@@ -586,7 +628,9 @@ class BaseResponse(_TemplateEnvironmentMixin, ActionAuthenticatorMixin):
                 response_headers["status"] = http_error.code  # type: ignore[assignment]
                 response = http_error.description, response_headers  # type: ignore[assignment]
 
-            if isinstance(response, str):
+            if isinstance(response, ActionResult):
+                status, headers, body = self.serialized(response)
+            elif isinstance(response, str):
                 status = 200
                 body = response
             else:
@@ -1133,6 +1177,8 @@ def to_str(value: Any, spec: Dict[str, Any]) -> str:
         return utcfromtimestamp(value).replace(tzinfo=datetime.timezone.utc).isoformat()
     elif vtype == "string":
         return str(value)
+    elif vtype == "structure":
+        return ""
     elif value is None:
         return "null"
     else:
